# this file ONLY contains graph_to_ideal, and is meant to be
# loaded from other files
# via the command load('func_graph_to_ideal.sage')
# see the file experiments.sagews for an example
#
# DESC: graph_to_ideal takes as input a graph (and related information)
#     and returns as output the associated perfect matching ideal
#
# INPUT:
#     g: a object of sage graph class
#
# OUTPUT:
#    P: a F_2[x_1,...x_e], a polynomial ring over F2 with one variable per edge
#    I: a list of the polynomials in the perfect matching ideal
#
def graph_to_ideal(g,o,r):
    #print "Entering graph_to_ideal... "
    #print "G: " + str(G)
    #print "num_vertices = " + str(g.order())
    #print "num_edges = " + str(g.size())

    if o==0 and r==0:
        # establish the ring containing the ideal (this will be returned)
        # number of variables is number of edges
        P = PolynomialRing(FiniteField(2), g.size(), 'x', order="lex") # over F2
        #P = PolynomialRing(QQ, g.size(), 'x', order="lex") # over the rationals
        x = P.gens();
        #print str(type(P))
        I = list(); # create an empty list
        #print "Constructing boolean polynomials, one per edge"
        # for each edge, create the polynomial eij^2 - eij
        for e in range(0, g.size()):
            I.append( x[e]^2 - x[e] );
            #end for
            #print "Ideal = " + str(I)
        #endif

    if o==0 and r==1:
            # establish the ring containing the ideal (this will be returned)
            # number of variables is number of edges
            #P = PolynomialRing(FiniteField(2), g.size(), 'x', order="lex") # over F2
        P = PolynomialRing(CC, g.size(), 'x', order="lex") # over the rationals
        x = P.gens();
            #print str(type(P))
        I = list(); # create an empty list
            #print "Constructing boolean polynomials, one per edge"
            # for each edge, create the polynomial eij^2 - eij
        for e in range(0, g.size()):
            I.append( x[e]^2 - x[e] );
            #end for
            #print "Ideal = " + str(I)
            #endif

    if o==1 and r==0:
            # establish the ring containing the ideal (this will be returned)
            # number of variables is number of edges
        P = PolynomialRing(FiniteField(2), g.size(), 'x', order="deglex") # over F2
            #P = PolynomialRing(QQ, g.size(), 'x', order="lex") # over the rationals
        x = P.gens();
            #print str(type(P))
        I = list(); # create an empty list
            #print "Constructing boolean polynomials, one per edge"
            # for each edge, create the polynomial eij^2 - eij
        for e in range(0, g.size()):
            I.append( x[e]^2 - x[e] );
            #end for
            #print "Ideal = " + str(I)
            #endif

    if o==1 and r==1:
            # establish the ring containing the ideal (this will be returned)
            # number of variables is number of edges
            #P = PolynomialRing(FiniteField(2), g.size(), 'x', order="lex") # over F2
        P = PolynomialRing(CC, g.size(), 'x', order="deglex") # over the rationals
        x = P.gens();
            #print str(type(P))
        I = list(); # create an empty list
            #print "Constructing boolean polynomials, one per edge"
            # for each edge, create the polynomial eij^2 - eij
        for e in range(0, g.size()):
            I.append( x[e]^2 - x[e] );
            #end for
            #print "Ideal = " + str(I)
            #endif
    if o==2 and r==0:
            # establish the ring containing the ideal (this will be returned)
            # number of variables is number of edges
        P = PolynomialRing(FiniteField(2), g.size(), 'x', order="degrevlex") # over F2
            #P = PolynomialRing(QQ, g.size(), 'x', order="lex") # over the rationals
        x = P.gens();
            #print str(type(P))
        I = list(); # create an empty list
            #print "Constructing boolean polynomials, one per edge"
            # for each edge, create the polynomial eij^2 - eij
        for e in range(0, g.size()):
            I.append( x[e]^2 - x[e] );
            #end for
            #print "Ideal = " + str(I)
            #endif

    if o==2 and r==1:
        # establish the ring containing the ideal (this will be returned)
            # number of variables is number of edges
            #P = PolynomialRing(FiniteField(2), g.size(), 'x', order="lex") # over F2
        P = PolynomialRing(CC, g.size(), 'x', order="degrevlex") # over the rationals
        x = P.gens();
            #print str(type(P))
        I = list(); # create an empty list
            #print "Constructing boolean polynomials, one per edge"
            # for each edge, create the polynomial eij^2 - eij
        for e in range(0, g.size()):
            I.append( x[e]^2 - x[e] );
            #end for
            #print "Ideal = " + str(I)
            #endif

    # the incidence matrix is a (# of vertices) x (# of edges matrix) 
    # where A(i,j) = 1 if vertex i is incident on edge j, and 0 otherwise
    A = g.incidence_matrix()
    #print str(A)
    # this is just for debugging
    debug = 1
    if debug == 1:
        count = 0;
        for e in g.edge_iterator():
            #print "x[" + str(count) + "]: " + str(e)
            count = count + 1;
        #end for
    #end if debug

    # construct the polynomial vi = 1 + sum_{j \in neighborhood(i)} x_ij
    for i in range(0, g.order()): # for each vertex in the graph
        f = -1;
        #print "Constructing polynomial for vertex = " + str(i)
        for e in range(0, g.size()): # find the neighbors
            #print "e = " + str(e)
            if A[i,e] == 1:
                # vertex i is incident on edge e
                f = f + x[e]
            #endif
        #end for edges
        #print "f = " + str(f)
        I.append(f); # append vertex polynomial to ideal
    #end for vertices

    # having constructed the vertex polynomials, now construct the edge polynomial
    for i in range(0, g.order()): # for each vertex in the graph
        for e in range(0, g.size()): # find the first edge incident on vertex i
            if A[i,e] == 1:
                # vertex i is incident on edge e
                e_base = e;
                for ee in range(e_base + 1, g.size()):
                    if A[i,ee] == 1: # find the next incident edge
                        I.append(x[e_base]*x[ee]);
                        #print "Adding edge pair (" + str(e_base) + ", " + str(ee) + ")"
                    #endif
                #end for ee
            #endif
        #end for edges
    #end for vertices

    # use the "dictionary" method return the variables
    return {'P':P, 'I':I}
#enddef graph_to_ideal